@@ -4,4 +4,4 @@ from .expression import ExpressionData, SpikeInData
 from .metadata import MetaData
 from .quality_control import MappingStatsData
 from .splicing import SplicingData, SpliceJunctionData, DownsampledSplicingData
-from .study import Study, StudyFactory
+from .study import Study
