@@ -1,5 +1,6 @@
 """
-Common operations performed on all kinds of data types
+Base data class for all data types. All data types in flotilla inherit from
+this, or a child object (like ExpressionData).
 """
 import sys
 
@@ -10,10 +11,9 @@ import seaborn as sns
 from sklearn.preprocessing import StandardScaler
 
 from ..compute.decomposition import DataFramePCA, DataFrameNMF
-# from ..compute.clustering import Cluster
-from ..compute.infotheory import binify
-from ..compute.predict import PredictorConfigManager, \
-    PredictorDataSetManager, CLASSIFIER
+from ..compute.infotheory import binify, cross_phenotype_jsd, jsd_df_to_2d
+from ..compute.predict import PredictorConfigManager, PredictorDataSetManager, \
+    CLASSIFIER
 from ..visualize.decomposition import DecompositionViz
 from ..visualize.generic import violinplot, nmf_space_transitions, \
     simple_twoway_scatter
@@ -135,6 +135,7 @@ class BaseData(object):
         -----
         Any cells not marked as "technical_outliers", "outliers" or "pooled"
         are considered as single-cell samples.
+
         """
         self.data = data
         self.data_original = self.data
@@ -320,15 +321,13 @@ class BaseData(object):
 
     @property
     def _var_cut(self):
-        """Variance values which are 2 standard deviations away the mean
-        variance"""
+        """Variance cutoff, 2 std devs away from mean variance"""
         return self.data.var().dropna().mean() + 2 * self.data.var() \
             .dropna().std()
 
     @property
     def variant(self):
-        """Genes whose variance among all cells is 2 standard deviations away
-        from the mean variance"""
+        """Features whose variance is 2 std devs away from mean variance"""
         return self.data.columns[self.data.var() > self._var_cut]
 
     # def drop_outliers(self, data, outliers):
@@ -406,14 +405,54 @@ class BaseData(object):
     #         pass
     #     elif 'samples'.startswith(between):
     #         pass
-    #
-    # def jsd(self):
-    #     """Jensen-Shannon divergence showing most varying measurements within a
-    #     celltype and between celltypes
-    #     """
-    #     raise NotImplementedError
 
-    # TODO.md: Specify dtypes in docstring
+    def jsd_df(self, groupby=None, n_iter=100, n_bins=10):
+        """Jensen-Shannon divergence of features across phenotypes
+
+        Parameters
+        ----------
+        groupby : mappable
+            A samples to phenotypes mapping
+        n_iter : int
+            Number of bootstrap resampling iterations to perform for the
+            within-group comparisons
+        n_bins : int
+            Number of bins to binify the singles data on
+
+        Returns
+        -------
+        jsd_df : pandas.DataFrame
+            A (n_features, n_phenotypes^2) dataframe of the JSD between each
+            feature between and within phenotypes
+        """
+        bins = np.linspace(self.singles.min().min(), self.singles.max().max(),
+                           n_bins)
+        return cross_phenotype_jsd(self.singles, groupby=groupby,
+                                   bins=bins, n_iter=n_iter)
+
+    def jsd_2d(self, groupby=None, n_iter=100, n_bins=10):
+        """Mean Jensen-Shannon divergence of features across phenotypes
+
+        Parameters
+        ----------
+        groupby : mappable
+            A samples to phenotypes mapping
+        n_iter : int
+            Number of bootstrap resampling iterations to perform for the
+            within-group comparisons
+        n_bins : int
+            Number of bins to binify the singles data on
+
+        Returns
+        -------
+        jsd_2d : pandas.DataFrame
+            A (n_phenotypes, n_phenotypes) symmetric dataframe of the mean JSD
+            between and within phenotypes
+        """
+        return jsd_df_to_2d(self.jsd_df(groupby=groupby, n_iter=n_iter,
+                                        n_bins=n_bins))
+
+
     def plot_classifier(self, trait, sample_ids=None, feature_ids=None,
                         predictor_name=None, standardize=True,
                         score_coefficient=None, data_name=None, groupby=None,
@@ -457,7 +496,8 @@ class BaseData(object):
 
         Returns
         -------
-        self : BaseData
+        cv : ClassifierViz
+            Visualziation of the classifier
         """
         # print trait
         plotting_kwargs = {} if plotting_kwargs is None else plotting_kwargs
@@ -478,8 +518,8 @@ class BaseData(object):
 
         if score_coefficient is not None:
             classifier.score_coefficient = score_coefficient
-        classifier(**plotting_kwargs)
-        return self
+        classifier.plot(**plotting_kwargs)
+        return classifier
 
     def plot_dimensionality_reduction(self, x_pc=1, y_pc=2, sample_ids=None,
                                       feature_ids=None, featurewise=False,
@@ -555,7 +595,7 @@ class BaseData(object):
                                       y_pc="pc_" + str(y_pc))
         # pca(show_vectors=True,
         # **plotting_kwargs)
-        return visualized(title=title,
+        return visualized.plot(title=title,
                           plot_violins=plot_violins, **plotting_kwargs)
 
     def plot_pca(self, **kwargs):
@@ -588,11 +628,13 @@ class BaseData(object):
         """
         if feature_ids is None:
             feature_ids = data.columns
+        else:
+            feature_ids = pd.Index(set(feature_ids).intersection(data.columns))
         if sample_ids is None:
             sample_ids = data.index
+        else:
+            sample_ids = pd.Index(set(sample_ids).intersection(data.index))
 
-        sample_ids = pd.Index(set(sample_ids).intersection(data.index))
-        feature_ids = pd.Index(set(feature_ids).intersection(data.columns))
 
         if len(sample_ids) == 1:
             sample_ids = sample_ids[0]
@@ -603,8 +645,8 @@ class BaseData(object):
         else:
             single_feature = False
 
-        subset = data.ix[sample_ids]
-        subset = subset.T.ix[feature_ids].T
+        subset = data.ix[sample_ids, feature_ids]
+        # subset = subset.T.ix[feature_ids].T
 
         if require_min_samples and not single_feature:
             subset = subset.ix[:, subset.count() >= self.minimum_samples]
@@ -615,9 +657,9 @@ class BaseData(object):
         return subset
 
     def _subset_singles_and_pooled(self, sample_ids=None,
-                                   feature_ids=None):
+                                   feature_ids=None, data=None):
         """Subset singles and pooled, taking only features that appear in both
-
+        
         Parameters
         ----------
         sample_ids : list-like, optional (default=None)
@@ -626,7 +668,12 @@ class BaseData(object):
             samples
         feature_ids : list-like, optional (default=None)
             List of feature ids to use. If None, use all
-
+        data : pandas.DataFrame, optional (default=None)
+            If provided, use this ``data`` instead of this instance's 
+            py:attr:`BaseData.data`. Convenient for when you filtered based on
+            some other criteria, e.g. for splicing events with expression 
+            greater than some threshold
+        
         Returns
         -------
         singles : pandas.DataFrame
@@ -636,19 +683,36 @@ class BaseData(object):
             DataFrame of only pooled samples, with only features that appear
             in both these single cell and pooled samples
         """
-        singles = self._subset(self.data, sample_ids, feature_ids,
-                               require_min_samples=True)
+        # singles_ids = self.data.index.intersection(sample_ids)
+        # pooled_ids = self.pooled.index.intersection(sample_ids)
+        # import pdb; pdb.set_trace()
+        if data is None:
+            singles = self._subset(self.singles, sample_ids, feature_ids,
+                                   require_min_samples=True)
+        else:
+            sample_ids = data.index.intersection(self.singles.index)
+            singles = self._subset(data, sample_ids,
+                                   require_min_samples=True)
+
         try:
-            # If the sample ids don't overlap with the pooled sample, assume
-            # you want all the pooled samples
-            if sample_ids is not None and sum(
-                    self.pooled.index.isin(sample_ids)) \
-                    > 0:
+            # If the sample ids don't overlap with the pooled sample, assume you
+            # want all the pooled samples
+            n_pooled_sample_ids = sum(self.pooled.index.isin(sample_ids))
+
+            if sample_ids is not None and n_pooled_sample_ids > 0:
                 pooled_sample_ids = sample_ids
             else:
                 pooled_sample_ids = None
-            pooled = self._subset(self.pooled, pooled_sample_ids, feature_ids,
-                                  require_min_samples=False)
+
+            if data is None:
+                pooled = self._subset(self.pooled, pooled_sample_ids,
+                                      feature_ids,
+                                      require_min_samples=False)
+            else:
+                sample_ids = data.index.intersection(self.pooled.index)
+                pooled = self._subset(data, sample_ids,
+                                      require_min_samples=False)
+
             if feature_ids is None or len(feature_ids) > 1:
                 # These are DataFrames
                 singles, pooled = singles.align(pooled, axis=1, join='inner')
@@ -661,6 +725,22 @@ class BaseData(object):
 
         return singles, pooled
 
+    # def _subset_ids_or_data(self, sample_ids, feature_ids, data,
+    #                            singles=False):
+    #     if data is None:
+    #         if singles:
+    #             data = self.singles
+    #         else:
+    #             data = self.data
+    #         return self._subset(data, sample_ids, feature_ids,
+    #                             require_min_samples=False)
+    #     else:
+    #         if feature_ids is not None and sample_ids is not None:
+    #             raise ValueError('Can only specify `sample_ids` and '
+    #                              '`feature_ids` or `data`, but not both.')
+    #         else:
+    #             return data
+
     def _subset_and_standardize(self, data, sample_ids=None,
                                 feature_ids=None,
                                 standardize=True, return_means=False,
@@ -785,7 +865,7 @@ class BaseData(object):
                               groupby=outlier_detector.outliers,
                               )
 
-        dv(show_point_labels=show_point_labels)
+        dv.plot(show_point_labels=show_point_labels)
         #self.plot_pca(self, groupby=outlier_detector.outliers,
         #              title=outlier_detector.title,
         #              **pca_args)
@@ -808,8 +888,8 @@ class BaseData(object):
     # @memoize
     def reduce(self, sample_ids=None, feature_ids=None,
                featurewise=False,
-               reducer=None,
-               standardize=None,
+               reducer=DataFramePCA,
+               standardize=True,
                reducer_kwargs=None, bins=None):
         """Make and memoize a reduced dimensionality representation of data
 
@@ -840,11 +920,6 @@ class BaseData(object):
         reducer_object : flotilla.compute.reduce.ReducerViz
             A ready-to-plot object containing the reduced space
         """
-        if reducer is None:
-            reducer = DataFramePCA
-
-        if standardize is None:
-            standardize = True
 
         reducer_kwargs = {} if reducer_kwargs is None else reducer_kwargs
 
@@ -863,6 +938,7 @@ class BaseData(object):
         reducer_object.means = means
         return reducer_object
 
+    @memoize
     def classify(self, trait, sample_ids, feature_ids,
                  standardize=True,
                  data_name='expression',
@@ -997,7 +1073,7 @@ class BaseData(object):
                      phenotype_groupby=None,
                      phenotype_order=None, color=None,
                      phenotype_to_color=None,
-                     phenotype_to_marker=None, xlabel=None, ylabel=None,
+                     phenotype_to_marker=None, nmf_xlabel=None, nmf_ylabel=None,
                      nmf_space=False, fig=None, axesgrid=None):
         """
         Plot the violinplot of a feature. Have the option to show NMF movement
@@ -1027,7 +1103,7 @@ class BaseData(object):
                              phenotype_groupby=phenotype_groupby,
                              phenotype_order=phenotype_order, ax=axes[0],
                              color=color)
-            # if self.data_type == 'splicing':
+
             if nmf_space:
                 try:
                     self.plot_nmf_space_transitions(
@@ -1035,10 +1111,11 @@ class BaseData(object):
                         phenotype_to_color=phenotype_to_color,
                         phenotype_to_marker=phenotype_to_marker,
                         order=phenotype_order, ax=axes[1],
-                        xlabel=xlabel, ylabel=ylabel)
+                        xlabel=nmf_xlabel, ylabel=nmf_ylabel)
                 except KeyError:
                     continue
             sns.despine()
+        fig.tight_layout()
 
     def nmf_space_positions(self, groupby, n=5):
         """Calculate NMF-space position of splicing events in phenotype groups
@@ -1171,7 +1248,7 @@ class BaseData(object):
         std = np.sqrt(np.square(nmf_space_transitions - mean).sum().sum() / n)
 
         big_transitions = nmf_space_transitions[
-            nmf_space_transitions > (mean + 2*std)].dropna(how='all')
+            nmf_space_transitions > (mean + 2 * std)].dropna(how='all')
         return big_transitions
 
     def plot_big_nmf_space_transitions(self, phenotype_groupby,
@@ -1216,7 +1293,8 @@ class BaseData(object):
                               nmf_space=True, fig=fig, axesgrid=axesgrid)
 
 
-    def plot_two_samples(self, sample1, sample2, **kwargs):
+    def plot_two_samples(self, sample1, sample2, fillna=None,
+                         **kwargs):
         """
 
         Parameters
@@ -1225,6 +1303,8 @@ class BaseData(object):
             Name of the sample to plot on the x-axis
         sample2 : str
             Name of the sample to plot on the y-axis
+        fillna : float
+            Value to replace NAs with
         Any other keyword arguments valid for seaborn.jointplot
 
         Returns
@@ -1239,10 +1319,14 @@ class BaseData(object):
         """
         x = self.data.ix[sample1]
         y = self.data.ix[sample2]
+
+        if fillna is not None:
+            x = x.fillna(fillna)
+            y = y.fillna(fillna)
         return simple_twoway_scatter(x, y, **kwargs)
 
     def plot_two_features(self, feature1, feature2, groupby=None,
-                          label_to_color=None, **kwargs):
+                          label_to_color=None, fillna=None, **kwargs):
         """Plot the values of two features
         """
         feature1s = self.maybe_renamed_to_feature_id(feature1)
@@ -1252,6 +1336,10 @@ class BaseData(object):
                 x = self.data.ix[:, f1].copy()
                 y = self.data.ix[:, f2].copy()
 
+                if fillna is not None:
+                    x = x.fillna(fillna)
+                    y = y.fillna(fillna)
+
                 x.name = feature1
                 y.name = feature2
                 x, y = x.align(y, 'inner')
@@ -1263,6 +1351,85 @@ class BaseData(object):
                                               for i in x.index]
                 simple_twoway_scatter(x, y, joint_kws=joint_kws, **kwargs)
 
+    @staticmethod
+    def _figsizer(shape, multiplier=0.25):
+        """Scale a heatmap figure based on the dataframe shape"""
+        return tuple(reversed(map(lambda x: min(x * multiplier, 40),
+                                  shape)))
+
+
+    def plot_clustermap(self, sample_ids=None, feature_ids=None, data=None,
+                        feature_colors=None, sample_id_to_color=None,
+                        metric='euclidean', method='average',
+                        scale_fig_by_data=True, **kwargs):
+        # data = self._subset_ids_or_data(sample_ids, feature_ids, data)
+        if data is None:
+            data = self._subset(self.data, sample_ids, feature_ids,
+                                require_min_samples=False)
+        # Get a mask of what values are NA, then replace them because
+        # clustering doesn't work if there's NAs
+        data = data.dropna(how='all', axis=1).dropna(how='all', axis=0)
+        mask = data.isnull()
+        data = data.fillna(data.mean())
+
+        if sample_id_to_color is not None:
+            sample_colors = [sample_id_to_color[x] for x in data.index]
+
+        col_colors = feature_colors
+        row_colors = sample_colors
+        data.columns = data.columns.map(self.feature_renamer)
+
+        if scale_fig_by_data:
+            figsize = self._figsizer(data.shape)
+            kwargs.pop('figsize')
+        else:
+            figsize = kwargs.pop('figsize', None)
+
+
+        return sns.clustermap(data, linewidth=0, col_colors=col_colors,
+                              row_colors=row_colors, metric=metric,
+                              method=method, figsize=figsize, mask=mask,
+                              **kwargs)
+
+    def plot_correlations(self, sample_ids=None, feature_ids=None, data=None,
+                          featurewise=False, sample_id_to_color=None,
+                          metric='euclidean', method='average',
+                          scale_fig_by_data=True, **kwargs):
+        if data is None:
+            data = self._subset(self.data, sample_ids, feature_ids,
+                                require_min_samples=False)
+
+        if sample_id_to_color is not None and not featurewise:
+            colors = [sample_id_to_color[x] for x in data.index]
+        else:
+            colors = None
+
+        if not featurewise:
+            data = data.T
+        corr = data.corr()
+        corr = corr.dropna(how='all', axis=0).dropna(how='all', axis=1)
+
+        # Get a mask of what values are NA, then replace them because
+        # clustering doesn't work if there's NAs
+        mask = corr.isnull()
+        corr = corr.fillna(data.mean())
+
+        if featurewise:
+            corr.index = corr.index.map(self.feature_renamer)
+            corr.columns = corr.columns.map(self.feature_renamer)
+
+        if scale_fig_by_data:
+            figsize = self._figsizer(corr.shape)
+            kwargs.pop('figsize')
+        else:
+            figsize = kwargs.pop('figsize', None)
+
+        return sns.clustermap(corr, linewidth=0, col_colors=colors,
+                              row_colors=colors, figsize=figsize,
+                              method=method, metric=metric, mask=mask,
+                              **kwargs)
+
+
 def subsets_from_metadata(metadata, minimum, subset_type, ignore=None):
     """
 
