@@ -17,13 +17,13 @@ from .expression import ExpressionData, SpikeInData
 from .quality_control import MappingStatsData, MIN_READS
 from .splicing import SplicingData, FRACTION_DIFF_THRESH
 from ..compute.predict import PredictorConfigManager
-from ..datapackage import data_package_url_to_dict, \
+from ..datapackage import datapackage_url_to_dict, \
     check_if_already_downloaded, make_study_datapackage
 from ..visualize.color import blue
 from ..visualize.ipython_interact import Interactive
 from ..datapackage import FLOTILLA_DOWNLOAD_DIR
 from ..util import load_csv, load_json, load_tsv, load_gzip_pickle_df, \
-    load_pickle_df, timestamp
+    load_pickle_df, timestamp, cached_property
 
 
 SPECIES_DATA_PACKAGE_BASE_URL = 'http://sauron.ucsd.edu/flotilla_projects'
@@ -72,6 +72,7 @@ class Study(object):
                  splicing_feature_data=None,
                  splicing_feature_rename_col=None,
                  splicing_feature_ignore_subset_cols=None,
+                 splicing_feature_expression_id_col=None,
                  mapping_stats_data=None,
                  mapping_stats_number_mapped_col=None,
                  mapping_stats_min_reads=MIN_READS,
@@ -144,6 +145,9 @@ class Study(object):
             example, if your splicing IDs are MISO IDs, but you want to plot
             Ensembl IDs, make sure the column you want, e.g. "ensembl_id" is
             in your dataframe and specify that. Default "gene_name".
+        splicing_feature_expression_id_col : str
+            A column name in the splicing_feature_data dataframe that
+            corresponds to the row names of the expression data
         mapping_stats_data : pandas.DataFrame
             Samples x feature dataframe of mapping stats measurements.
             Currently, this
@@ -241,7 +245,7 @@ class Study(object):
                 predictor_config_manager=self.predictor_config_manager,
                 min_reads=mapping_stats_min_reads)
             self.technical_outliers = self.mapping_stats.too_few_mapped
-            sys.stderr.write('samples had too few mapped reads (<{'
+            sys.stderr.write('Samples had too few mapped reads (<{'
                              ':.1e} reads):\n\t{}\n'.format(
                 mapping_stats_min_reads, ', '.join(self.technical_outliers)))
         else:
@@ -305,7 +309,8 @@ class Study(object):
                 predictor_config_manager=self.predictor_config_manager,
                 technical_outliers=self.technical_outliers,
                 minimum_samples=metadata_minimum_samples,
-                feature_ignore_subset_cols=splicing_feature_ignore_subset_cols)
+                feature_ignore_subset_cols=splicing_feature_ignore_subset_cols,
+                feature_expression_id_col=splicing_feature_expression_id_col)
 
         if spikein_data is not None:
             self.spikein = SpikeInData(
@@ -346,7 +351,7 @@ class Study(object):
     def from_datapackage_url(
             cls, datapackage_url,
             load_species_data=True,
-            species_data_package_base_url=SPECIES_DATA_PACKAGE_BASE_URL):
+            species_datapackage_base_url=SPECIES_DATA_PACKAGE_BASE_URL):
         """Create a study from a url of a datapackage.json file
 
         Parameters
@@ -372,10 +377,13 @@ class Study(object):
             If the datapackage.json file does not contain the required
             resources of metadata, expression, and splicing.
         """
-        data_package = data_package_url_to_dict(datapackage_url)
+        datapackage = datapackage_url_to_dict(datapackage_url)
+        datapackage_dir = '{}/{}'.format(FLOTILLA_DOWNLOAD_DIR, 
+                                         datapackage['name'])
         return cls.from_datapackage(
-            data_package, load_species_data=load_species_data,
-            species_datapackage_base_url=species_data_package_base_url)
+            datapackage, load_species_data=load_species_data,
+            datapackage_dir=datapackage_dir,
+            species_datapackage_base_url=species_datapackage_base_url)
 
     @classmethod
     def from_datapackage_file(
@@ -392,6 +400,10 @@ class Study(object):
             load_species_data=load_species_data,
             species_datapackage_base_url=species_datapackage_base_url)
 
+    @staticmethod
+    def _is_absolute_path(location):
+        return location.startswith('http') or location.startswith('/')
+
     @classmethod
     def from_datapackage(
             cls, datapackage, datapackage_dir='./',
@@ -418,6 +430,9 @@ class Study(object):
         for resource in datapackage['resources']:
             if 'url' in resource:
                 resource_url = resource['url']
+                if not cls._is_absolute_path(resource_url):
+                    resource_url = '{}/{}'.format(datapackage_dir,
+                                                  resource_url)
                 filename = check_if_already_downloaded(resource_url,
                                                        datapackage_name)
             else:
@@ -426,6 +441,10 @@ class Study(object):
                                                            datapackage_name)
                 else:
                     filename = resource['path']
+                    if not cls._is_absolute_path(filename):
+                        filename = '{}/{}'.format(datapackage_dir,
+                                                  filename)
+
                     # Test if the file exists, if not, then add the datapackage
                     # file
                     try:
@@ -444,9 +463,6 @@ class Study(object):
             dfs[name] = reader(filename, compression=compression,
                                header=header)
 
-            if name == 'expression':
-                if 'log_transformed' in resource:
-                    log_base = 2
             for key in set(resource.keys()).difference(
                     DATAPACKAGE_RESOURCE_COMMON_KWS):
                 kwargs['{}_{}'.format(name, key)] = resource[key]
@@ -502,10 +518,10 @@ class Study(object):
         try:
             species_data_url = '{}/{}/datapackage.json'.format(
                 species_datapackage_base_url, species)
-            species_data_package = data_package_url_to_dict(
+            species_datapackage = datapackage_url_to_dict(
                 species_data_url)
 
-            for resource in species_data_package['resources']:
+            for resource in species_datapackage['resources']:
                 if 'url' in resource:
                     resource_url = resource['url']
                     filename = check_if_already_downloaded(resource_url,
@@ -552,21 +568,17 @@ class Study(object):
         feature_ids = self.feature_subset_to_feature_ids(data_type,
                                                          feature_subset,
                                                          rename=False)
-
         if data_type == "expression":
-            obj = self.expression
+            datamodel = self.expression
         elif data_type == "splicing":
-            obj = self.splicing
+            datamodel = self.splicing
 
-        reducer, outlier_detector = obj.detect_outliers(sample_ids=sample_ids,
-                                                        feature_ids=feature_ids,
-                                                        featurewise=featurewise,
-                                                        reducer=reducer,
-                                                        standardize=standardize,
-                                                        reducer_kwargs=reducer_kwargs,
-                                                        bins=bins,
-                                                        outlier_detection_method=outlier_detection_method,
-                                                        outlier_detection_method_kwargs=outlier_detection_method_kwargs)
+        reducer, outlier_detector = datamodel.detect_outliers(
+            sample_ids=sample_ids, feature_ids=feature_ids,
+            featurewise=featurewise, reducer=reducer, standardize=standardize,
+            reducer_kwargs=reducer_kwargs, bins=bins,
+            outlier_detection_method=outlier_detection_method,
+            outlier_detection_method_kwargs=outlier_detection_method_kwargs)
 
         outlier_detector.predict(reducer.reduced_space)
         outlier_detector.title = "_".join(
@@ -581,8 +593,7 @@ class Study(object):
         return reducer, outlier_detector
 
     def drop_outliers(self):
-        """remove samples labeled "outlier" in self.metadata,
-        replace the data in self.expression and self.splicing with the smaller version"""
+        """Assign samples marked as "outlier" in metadata, to other datas"""
         outliers = self.metadata.data['outlier'][
             self.metadata.data['outlier']].index
         self.expression.outlier_samples = outliers
@@ -657,11 +668,17 @@ class Study(object):
             List of sample ids in the data
         """
 
+        # IF this is a list of IDs
+
         try:
             return self.metadata.sample_subsets[phenotype_subset]
         except KeyError:
             pass
 
+        ind = self.metadata.phenotype_series == phenotype_subset
+        if ind.sum() > 0:
+            return self.metadata.phenotype_series.index[ind]
+
         try:
             if phenotype_subset is None or 'all_samples'.startswith(
                     phenotype_subset):
@@ -682,7 +699,7 @@ class Study(object):
 
     def plot_pca(self, data_type='expression', x_pc=1, y_pc=2,
                  sample_subset=None, feature_subset=None,
-                 title='', featurewise=False, plot_violins=True,
+                 title='', featurewise=False, plot_violins=False,
                  show_point_labels=False, reduce_kwargs=None,
                  **kwargs):
         """Performs DataFramePCA on both expression and splicing study_data
@@ -858,6 +875,8 @@ class Study(object):
             raise ValueError("All samples are True (or all samples are "
                              "False) or all are the same, cannot classify"
                              "when all samples are the same")
+        trait_data = pd.Series(trait_data, name=trait,
+                               index=self.metadata.data.index)
 
         sample_ids = self.sample_subset_to_sample_ids(sample_subset)
         feature_ids = self.feature_subset_to_feature_ids(data_type,
@@ -902,23 +921,123 @@ class Study(object):
         elif data_type == "splicing":
             self.splicing.plot_regressor(**kwargs)
 
-    def modalities(self, sample_subset=None, feature_subset=None):
-        """Get modality assignments of
+    def modalities(self, sample_subset=None, feature_subset=None,
+                   expression_thresh=-np.inf, bootstrapped=False,
+                   bootstrapped_kws=None):
+        """Get splicing modality assignments of data
+
+        Parameters
+        ----------
+        sample_subset : str or None, optional
+            Which subset of the samples to use, based on some phenotype
+            column in the experiment design data. If None, all samples are
+            used.
+        feature_subset : str or None, optional
+            Which subset of the features to used, based on some feature type
+            in the expression data (e.g. "variant"). If None, all features
+            are used.
+        expression_thresh : float, optional
+            Minimum expression value, of the original input. E.g. if the
+            original input is already log-transformed, then this threshold is
+            on the log values.
+        bootstrapped : bool, optional
+            Whether or not to use bootstrap resampling of each splicing event
+            to robustly estimate its modality
+        bootstrapped_kws : dict, optional
+            Valid arguments to _bootstrapped_fit_transform. If None, default is
+            dict(n_iter=100, thresh=0.6, minimum_samples=10)
+
+        Returns
+        -------
+        modalities : pandas.Series
+            A (n_events,) shaped series of the assigned modality (in the case
+            of bootstrapped=False), or modality most commonly assigned, in the
+            case of bootstrapped=True
 
         """
-        sample_ids = self.sample_subset_to_sample_ids(sample_subset)
-        feature_ids = self.feature_subset_to_feature_ids('splicing',
-                                                         feature_subset,
-                                                         rename=False)
-        return self.splicing.modalities(sample_ids, feature_ids)
+        if expression_thresh > -np.inf:
+            data = self.filter_splicing_on_expression(
+                expression_thresh=expression_thresh,
+                sample_subset=sample_subset)
+            sample_ids = None
+            feature_ids = None
+        else:
+            sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+            feature_ids = self.feature_subset_to_feature_ids(
+                'splicing', feature_subset, rename=False)
+            data = None
+
+        return self.splicing.modalities(sample_ids, feature_ids, data=data,
+                                        bootstrapped=bootstrapped,
+                                        groupby=self.sample_id_to_phenotype,
+                                        bootstrapped_kws=bootstrapped_kws)
+
+    def modalities_counts(self, sample_subset=None, feature_subset=None,
+                   expression_thresh=-np.inf, bootstrapped=True,
+                   bootstrapped_kws=None):
+        """Get counts of each resampled splicing event assigned to a modality
+
+        Parameters
+        ----------
+        sample_subset : str or None, optional
+            Which subset of the samples to use, based on some phenotype
+            column in the experiment design data. If None, all samples are
+            used.
+        feature_subset : str or None, optional
+            Which subset of the features to used, based on some feature type
+            in the expression data (e.g. "variant"). If None, all features
+            are used.
+        expression_thresh : float, optional
+            Minimum expression value, of the original input. E.g. if the
+            original input is already log-transformed, then this threshold is
+            on the log values.
+        bootstrapped : bool, optional
+            Whether or not to use bootstrap resampling of each splicing event
+            to robustly estimate its modality
+        bootstrapped_kws : dict, optional
+            Valid arguments to _bootstrapped_fit_transform. If None, default is
+            dict(n_iter=100, thresh=0.6, minimum_samples=10)
+
+        Returns
+        -------
+        modalities : pandas.Series
+            A (n_events,) shaped series of the assigned modality (in the case
+            of bootstrapped=False), or modality most commonly assigned, in the
+            case of bootstrapped=True
+
+        """
+        if expression_thresh > -np.inf:
+            data = self.filter_splicing_on_expression(
+                expression_thresh=expression_thresh,
+                sample_subset=sample_subset)
+            sample_ids = None
+            feature_ids = None
+        else:
+            sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+            feature_ids = self.feature_subset_to_feature_ids(
+                'splicing', feature_subset, rename=False)
+            data = None
+
+        return self.splicing.modalities_counts(
+            sample_ids, feature_ids, data=data, bootstrapped=bootstrapped,
+            bootstrapped_kws=bootstrapped_kws,
+            groupby=self.sample_id_to_phenotype)
 
     def plot_modalities(self, sample_subset=None, feature_subset=None,
-                        normed=True):
+                        normed=True, bootstrapped=False,
+                        bootstrapped_kws=None, expression_thresh=-np.inf):
         # try:
-        sample_ids = self.sample_subset_to_sample_ids(sample_subset)
-        feature_ids = self.feature_subset_to_feature_ids('splicing',
-                                                         feature_subset,
-                                                         rename=False)
+        if expression_thresh > -np.inf:
+            data = self.filter_splicing_on_expression(
+                expression_thresh=expression_thresh,
+                sample_subset=sample_subset)
+            sample_ids = None
+            feature_ids = None
+        else:
+            sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+            feature_ids = self.feature_subset_to_feature_ids(
+                'splicing', feature_subset, rename=False)
+            data = None
 
         grouped = self.sample_id_to_phenotype.groupby(
             self.sample_id_to_phenotype)
@@ -930,10 +1049,14 @@ class Study(object):
         bar_ax = axes[0]
         all_ax = axes[1]
         self.splicing.plot_modalities_reduced(sample_ids, feature_ids,
-                                              all_ax, title='all samples')
-        self.splicing.plot_modalities_bar(sample_ids, feature_ids,
+                                              all_ax, title='all samples',
+                                              bootstrapped=bootstrapped,
+                                              bootstrapped_kws=bootstrapped_kws)
+        self.splicing.plot_modalities_stacked_bar(sample_ids, feature_ids,
                                           bar_ax, i=0, normed=normed,
-                                          legend=False)
+                                          legend=False,
+                                          bootstrapped=bootstrapped,
+                                          bootstrapped_kws=bootstrapped_kws)
 
         axes = axes[2:]
         for i, ((celltype, series), ax) in enumerate(zip(grouped, axes)):
@@ -941,7 +1064,7 @@ class Study(object):
             sys.stdout.write('\n---- {} ----\n'.format(celltype))
             samples = series.index.intersection(sample_ids)
             # legend = i == 0
-            self.splicing.plot_modalities_bar(samples, feature_ids,
+            self.splicing.plot_modalities_stacked_bar(samples, feature_ids,
                                               bar_ax, i + 1, normed=normed,
                                               legend=False)
 
@@ -979,43 +1102,54 @@ class Study(object):
         n_total = self.celltype_event_counts.sum(axis=1).astype(float)
         return n_unique / n_total * 100
 
-    @property
-    def celltype_modalities(self):
-        """Return modality assignments of each celltype
-        """
-        return self.splicing.data.groupby(
-            self.sample_id_to_phenotype, axis=0).apply(
-            lambda x: self.splicing.modalities(x.index))
+    # @property
+    # def celltype_modalities(self):
+    #     """Return modality assignments of each celltype
+    #     """
+    #     return self.splicing.data.groupby(
+    #         self.sample_id_to_phenotype, axis=0).apply(
+    #         lambda x: self.splicing.modalities(x.index))
 
     def plot_modalities_lavalamps(self, sample_subset=None, bootstrapped=False,
-                                  bootstrapped_kws=None):
-        grouped = self.splicing.data.groupby(self.sample_id_to_color, axis=0)
-        celltype_groups = self.splicing.data.groupby(
-            self.sample_id_to_phenotype, axis=0)
+                                  bootstrapped_kws=None,
+                                  expression_thresh=-np.inf):
+        # grouped = self.splicing.singles.groupby(self.sample_id_to_phenotype,
+        #                                         axis=0)
+        # celltype_groups = self.splicing.singles.groupby(
+        #     self.sample_id_to_phenotype, axis=0)
+        #
+        # if sample_subset is not None:
+        #     # Only plotting one sample_subset, use the modality assignments
+        #     # from just the samples from this sample_subset
+        #     celltype_samples = set(celltype_groups.groups[sample_subset])
+        # else:
+        #     # Plotting all the celltypes, use the modality assignments from
+        #     # all celltypes together
+        #     celltype_samples = self.splicing.data.index
 
-        if sample_subset is not None:
-            # Only plotting one sample_subset, use the modality assignments
-            # from just the samples from this sample_subset
-            celltype_samples = celltype_groups.groups[sample_subset]
-            celltype_samples = set(celltype_groups.groups[sample_subset])
-            use_these_modalities = True
+        if expression_thresh > -np.inf:
+            data = self.filter_splicing_on_expression(
+                expression_thresh=expression_thresh,
+                sample_subset=sample_subset)
+            sample_ids = None
         else:
-            # Plotting all the celltypes, use the modality assignments from
-            # all celltypes together
-            celltype_samples = self.splicing.data.index
-            use_these_modalities = False
+            sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+            data = None
 
-        for i, (color, sample_ids) in enumerate(grouped.groups.iteritems()):
-            x_offset = 1. / (i + 1)
-            sample_ids = celltype_samples.intersection(sample_ids)
-            if len(sample_ids) > 0:
-                self.splicing.plot_modalities_lavalamps(
-                    sample_ids=sample_ids,
-                    color=color,
-                    x_offset=x_offset,
-                    use_these_modalities=use_these_modalities,
-                    bootstrapped=bootstrapped,
-                    bootstrapped_kws=bootstrapped_kws)
+        # for i, (phenotype, sample_ids) in enumerate(grouped.groups.iteritems()):
+        #     x_offset = 1. / (i + 1)
+        #     sample_ids = celltype_samples.intersection(sample_ids)
+        #     color = self.phenotype_to_color[phenotype]
+        #     if len(sample_ids) > 0:
+        self.splicing.plot_modalities_lavalamps(
+            groupby=self.sample_id_to_phenotype,
+            phenotype_to_color=self.phenotype_to_color,
+            sample_ids=sample_ids, data=data,
+            # color=color,
+            # x_offset=x_offset,
+            # title=phenotype,
+            bootstrapped=bootstrapped,
+            bootstrapped_kws=bootstrapped_kws)
 
     def plot_event(self, feature_id, sample_subset=None, nmf_space=False):
         """Plot the violinplot and NMF transitions of a splicing event
@@ -1040,8 +1174,9 @@ class Study(object):
             phenotype_to_marker=self.phenotype_to_marker, nmf_space=nmf_space)
 
     def plot_lavalamp_pooled_inconsistent(
-            self, sample_subset=None, feature_ids=None,
-            fraction_diff_thresh=FRACTION_DIFF_THRESH):
+            self, sample_subset=None, feature_subset=None,
+            fraction_diff_thresh=FRACTION_DIFF_THRESH,
+            expression_thresh=-np.inf):
         # grouped_ids = self.splicing.data.groupby(self.sample_id_to_color,
         # axis=0)
         celltype_groups = self.metadata.data.groupby(
@@ -1054,9 +1189,11 @@ class Study(object):
             # Plotting all the celltypes
             celltype_samples = self.sample_subset_to_sample_ids(sample_subset)
 
+        feature_ids = self.feature_subset_to_feature_ids('splicing',
+                                                            feature_subset=feature_subset)
+
         celltype_and_sample_ids = celltype_groups.groups.iteritems()
-        for i, (phenotype, sample_ids) in enumerate(
-                celltype_and_sample_ids):
+        for i, (phenotype, sample_ids) in enumerate(celltype_and_sample_ids):
             # import pdb; pdb.set_trace()
 
             # Assumes all samples of a sample_subset have the same color...
@@ -1065,14 +1202,15 @@ class Study(object):
             sample_ids = celltype_samples.intersection(sample_ids)
             if len(sample_ids) == 0:
                 continue
-            self.splicing.plot_lavalamp_pooled_inconsistent(
-                sample_ids, feature_ids, fraction_diff_thresh,
-                color=color)
-
-    def percent_pooled_inconsistent(self,
-                                    sample_subset=None, feature_ids=None,
-                                    fraction_diff_thresh=FRACTION_DIFF_THRESH):
+            data = self.filter_splicing_on_expression(expression_thresh)
+            data = data.ix[sample_ids, :]
+            self.splicing.plot_lavalamp_pooled_inconsistent(data,
+                feature_ids, fraction_diff_thresh, color=color)
 
+    def percent_pooled_inconsistent(
+            self, sample_subset=None, feature_subset=None,
+            fraction_diff_thresh=FRACTION_DIFF_THRESH,
+            expression_thresh=-np.inf):
         celltype_groups = self.metadata.data.groupby(
             self.sample_id_to_phenotype, axis=0)
 
@@ -1081,57 +1219,135 @@ class Study(object):
             celltype_samples = set(celltype_groups.groups[sample_subset])
         else:
             # Plotting all the celltypes
-            celltype_samples = self.metadata.data.index
+            celltype_samples = self.sample_subset_to_sample_ids(sample_subset)
+
+        feature_ids = self.feature_subset_to_feature_ids('splicing',
+                                                         feature_subset=feature_subset)
 
         celltype_and_sample_ids = celltype_groups.groups.iteritems()
-        for i, (sample_subset, sample_ids) in enumerate(
-                celltype_and_sample_ids):
+        percents = pd.Series(celltype_groups.groups.keys())
+        for i, (phenotype, sample_ids) in enumerate(celltype_and_sample_ids):
             # import pdb; pdb.set_trace()
 
             # Assumes all samples of a sample_subset have the same color...
             # probably wrong
-            color = self.sample_id_to_color[sample_ids[0]]
+            color = self.phenotype_to_color[phenotype]
             sample_ids = celltype_samples.intersection(sample_ids)
             if len(sample_ids) == 0:
                 continue
-            self.splicing.percent_pooled_inconsistent(sample_ids, feature_ids,
-                                                      fraction_diff_thresh)
+            data = self.filter_splicing_on_expression(expression_thresh)
+            data = data.ix[sample_ids, :]
+            singles, pooled, not_measured_in_pooled, pooled_inconsistent =\
+                self.splicing.pooled_inconsistent(data, feature_ids,
+                                              fraction_diff_thresh)
+            percent = self._percent_pooled_inconsistent(pooled,
+                                                        pooled_inconsistent)
+            percents[phenotype] = percent
+        return percents
 
-    # def plot_clusteredheatmap(self, sample_subset=None,
-    # feature_subset='variant',
-    # data_type='expression', metric='euclidean',
-    # linkage_method='median', figsize=None):
-    # if data_type == 'expression':
-    # data = self.expression.data
-    #     elif data_type == 'splicing':
-    #         data = self.splicing.data
-    #     celltype_groups = data.groupby(
-    #         self.sample_id_to_phenotype, axis=0)
-    #
-    #     if sample_subset is not None:
-    #         # Only plotting one sample_subset
-    #         try:
-    #             sample_ids = set(celltype_groups.groups[sample_subset])
-    #         except KeyError:
-    #             sample_ids = self.sample_subset_to_sample_ids(sample_subset)
-    #     else:
-    #         # Plotting all the celltypes
-    #         sample_ids = data.index
-    #
-    #     sample_colors = [self.sample_id_to_color[x] for x in sample_ids]
-    #     feature_ids = self.feature_subset_to_feature_ids(data_type,
-    #                                                      feature_subset,
-    #                                                      rename=False)
-    #
-    #     if data_type == "expression":
-    #         return self.expression.plot_clusteredheatmap(
-    #             sample_ids, feature_ids, linkage_method=linkage_method,
-    #             metric=metric, sample_colors=sample_colors, figsize=figsize)
-    #     elif data_type == "splicing":
-    #         return self.splicing.plot_clusteredheatmap(
-    #             sample_ids, feature_ids, linkage_method=linkage_method,
-    #             metric=metric, sample_colors=sample_colors, figsize=figsize)
 
+    def plot_clustermap(self, sample_subset=None, feature_subset=None,
+                        data_type='expression', metric='euclidean',
+                        method='average', figsize=None,
+                        scale_fig_by_data=True, **kwargs):
+        """Visualize hierarchical relationships within samples and features
+
+        Parameters
+        ----------
+
+
+        Returns
+        -------
+
+
+        Raises
+        ------
+        """
+
+        if feature_subset is None:
+            feature_subset = self.default_feature_subset
+
+        sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+        feature_ids = self.feature_subset_to_feature_ids(data_type,
+                                                         feature_subset,
+                                                         rename=False)
+
+        if data_type == "expression":
+            return self.expression.plot_clustermap(
+                sample_ids=sample_ids, feature_ids=feature_ids, method=method,
+                metric=metric, sample_id_to_color=self.sample_id_to_color,
+                figsize=figsize, scale_fig_by_data=scale_fig_by_data,
+                **kwargs)
+        elif data_type == "splicing":
+            return self.splicing.plot_clustermap(
+                sample_ids=sample_ids, feature_ids=feature_ids, method=method,
+                metric=metric, sample_id_to_color=self.sample_id_to_color,
+                figsize=figsize, scale_fig_by_data=scale_fig_by_data,
+                **kwargs)
+
+    def plot_correlations(self, sample_subset=None, feature_subset=None,
+                        data_type='expression', metric='euclidean',
+                        method='average', figsize=None, featurewise=False,
+                        scale_fig_by_data=True, **kwargs):
+        """Visualize clustered correlations of samples across features
+
+        Parameters
+        ----------
+
+
+        Returns
+        -------
+
+
+        Raises
+        ------
+
+        """
+
+        if feature_subset is None:
+            feature_subset = self.default_feature_subset
+
+        sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+        feature_ids = self.feature_subset_to_feature_ids(data_type,
+                                                         feature_subset,
+                                                         rename=False)
+
+        if figsize is not None and scale_fig_by_data:
+            raise ValueError('If "scale_fig_by_data" is true, then cannot '
+                             'also specify "figsize"')
+
+        if data_type == "expression":
+            return self.expression.plot_correlations(
+                sample_ids=sample_ids, feature_ids=feature_ids,
+                sample_id_to_color=self.sample_id_to_color,
+                figsize=figsize, scale_fig_by_data=scale_fig_by_data,
+                metric=metric, method=method, featurewise=featurewise,
+                **kwargs)
+        elif data_type == "splicing":
+            return self.splicing.plot_correlations(
+                sample_ids=sample_ids, feature_ids=feature_ids, method=method,
+                metric=metric, sample_id_to_color=self.sample_id_to_color,
+                figsize=figsize, scale_fig_by_data=scale_fig_by_data,
+                featurewise=featurewise, **kwargs)
+
+    def plot_lavalamps(self, sample_subset=None, feature_subset=None,
+                       expression_thresh=-np.inf):
+        if expression_thresh > -np.inf:
+            data = self.filter_splicing_on_expression(
+                expression_thresh=expression_thresh,
+                sample_subset=sample_subset)
+            sample_ids = None
+            feature_ids = None
+        else:
+            sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+            feature_ids = self.feature_subset_to_feature_ids(
+                'splicing', feature_subset, rename=False)
+            data = None
+
+        self.splicing.plot_lavalamp(sample_ids, feature_ids, data,
+                                    self.sample_id_to_phenotype,
+                                    self.phenotype_to_color,
+                                    order=self.phenotype_order)
 
     def plot_big_nmf_space_transitions(self, data_type='expression', n=5):
         if data_type == 'expression':
@@ -1285,7 +1501,8 @@ class Study(object):
             expression = self.expression.data_original
             expression_kws = {
                 'log_base': self.expression.log_base,
-                'thresh': self.expression.thresh}
+                'thresh': self.expression.thresh,
+                'plus_one': self.expression.plus_one}
         except AttributeError:
             expression = None
             expression_kws = None
@@ -1312,7 +1529,9 @@ class Study(object):
             splicing_feature_kws = {'rename_col':
                                         self.splicing.feature_rename_col,
                                     'ignore_subset_cols':
-                                        self.splicing.feature_ignore_subset_cols}
+                                        self.splicing.feature_ignore_subset_cols,
+                                    'expression_id_col':
+                                        self.splicing.feature_expression_id_col}
         except AttributeError:
             splicing_feature_data = None
             splicing_feature_kws = None
@@ -1353,6 +1572,81 @@ class Study(object):
                                       version=version,
                                       flotilla_dir=flotilla_dir)
 
+    @cached_property()
+    def tidy_splicing_with_expression(self):
+        """A tall 'tidy' dataframe of samples with expression and splicing
+
+        :return:
+        :rtype:
+        """
+        # Tidify splicing
+        splicing = self.splicing.data
+        splicing_index_name = splicing.index.name
+        splicing_index_name = 'index' if splicing_index_name is None \
+            else splicing_index_name
+        if isinstance(splicing.columns, pd.MultiIndex):
+            splicing_columns_name = splicing.columns.names
+        else:
+            splicing_columns_name = splicing.columns.name
+
+        splicing_tidy = pd.melt(splicing.reset_index(),
+                                id_vars=splicing_index_name,
+                                value_name='psi',
+                                var_name=splicing_columns_name)
+        splicing_tidy = splicing_tidy.rename(columns={
+            splicing_index_name: 'sample_id'})
+        splicing_tidy['common_id'] = splicing_tidy[splicing_columns_name].map(
+            self.splicing.feature_data[self.splicing.feature_expression_id_col])
+
+        splicing_tidy = splicing_tidy.dropna()
+
+        # Tidify expression
+        expression = self.expression.data_original
+        expression_index_name = expression.index.name
+        expression_index_name = 'index' if expression_index_name is None \
+            else expression_index_name
+
+        expression_tidy = pd.melt(expression.reset_index(),
+                                  id_vars=expression_index_name,
+                                  value_name='expression',
+                                  var_name='common_id')
+        # if expression_index_name == 'index':
+        expression_tidy = expression_tidy.rename(columns={'index': 'sample_id'})
+        expression_tidy = expression_tidy.dropna()
+
+        splicing_tidy.set_index(['sample_id', 'common_id'], inplace=True)
+        expression_tidy.set_index(['sample_id', 'common_id'], inplace=True)
+
+        return splicing_tidy.join(expression_tidy, how='inner').reset_index()
+
+    def filter_splicing_on_expression(self, expression_thresh,
+                                      sample_subset=None):
+        """Filter splicing events on expression values
+
+        Parameters
+        ----------
+        expression_thresh : float
+            Minimum expression value, of the original input. E.g. if the
+            original input is already log-transformed, then this threshold is
+            on the log values.
+
+        Returns
+        -------
+        psi : pandas.DataFrame
+            A (n_samples, n_features)
+
+        """
+        sample_ids = self.sample_subset_to_sample_ids(sample_subset)
+        splicing_with_expression = \
+            self.tidy_splicing_with_expression.ix[
+                self.tidy_splicing_with_expression.sample_id.isin(sample_ids)]
+        ind = splicing_with_expression.expression >= expression_thresh
+        splicing_high_expression = splicing_with_expression.ix[ind]
+        splicing_high_expression = splicing_high_expression.reset_index().dropna()
+        filtered_psi = splicing_high_expression.pivot(
+            columns='miso_id', index='sample_id', values='psi')
+        return filtered_psi
+
 
 
 # Add interactive visualizations
@@ -1364,4 +1658,5 @@ Study.interactive_lavalamp_pooled_inconsistent = \
     Interactive.interactive_lavalamp_pooled_inconsistent
 Study.interactive_choose_outliers = Interactive.interactive_choose_outliers
 Study.interactive_reset_outliers = Interactive.interactive_reset_outliers
-# Study.interactive_clusteredheatmap = Interactive.interactive_clusteredheatmap
+Study.interactive_clustermap = Interactive.interactive_clustermap
+Study.interactive_correlations = Interactive.interactive_correlations
