@@ -8,7 +8,7 @@ import numpy as np
 import pandas as pd
 import seaborn as sns
 
-from .color import dark2
+from .color import dark2, deep
 from .generic import violinplot
 
 
@@ -191,7 +191,7 @@ class DecompositionViz(object):
             self.pc_loadings_labels[pc] = labels
             self.top_features.update(labels)
 
-    def __call__(self, ax=None, title='', plot_violins=False,
+    def plot(self, ax=None, title='', plot_violins=False,
                  show_point_labels=False,
                  show_vectors=True,
                  show_vector_labels=True,
@@ -225,6 +225,7 @@ class DecompositionViz(object):
             self.plot_violins()
         return self
 
+
     def shorten(self, x):
         if len(x) > self.max_char_width:
             return '{}...'.format(x[:self.max_char_width])
@@ -346,7 +347,12 @@ class DecompositionViz(object):
         if ax is None:
             ax = plt.gca()
 
-        ax.plot(loadings, np.arange(loadings.shape[0]), 'o')
+        # import pdb; pdb.set_trace()
+        x = loadings
+        y = np.arange(loadings.shape[0])
+
+        ax.scatter(x, y, color=deep[0])
+        ax.set_ylim(-.5, y.max()+.5)
 
         ax.set_yticks(np.arange(max(loadings.shape[0], n_features)))
         ax.set_title("Component " + pc)
@@ -418,7 +424,9 @@ class DecompositionViz(object):
                 else None
 
             if isinstance(feature_id, tuple):
-                feature_id = feature_id[0][:20]
+                feature_id = feature_id[0]
+            if len(feature_id) > 25:
+                feature_id = feature_id[:25] + '...'
             if renamed != feature_id:
                 title = '{}\n{}'.format(feature_id, renamed)
             else:
