@@ -17,8 +17,7 @@ from .decomposition import DecompositionViz
 class PredictorBaseViz(PredictorBase):
     _reducer_plotting_args = {}
 
-    def __call__(self, **pca_plotting_kwargs):
-
+    def plot(self, **pca_plotting_kwargs):
         if not self.has_been_fit:
             self.fit()
 
@@ -145,7 +144,7 @@ class PredictorBaseViz(PredictorBase):
                                   # label_to_color=self.label_to_color,
                                   # label_to_marker=self.label_to_marker,
         )
-        pcaviz(**local_plotting_kwargs)
+        pcaviz.plot(**local_plotting_kwargs)
         return pcaviz
 
     def plot_scores(self, ax=None):
